/* $Id: ncat_connect.h 12705 2009-03-26 23:15:49Z david $ */

#include "nsock.h"

/* handle nsock-powered connections */
extern void connect_evt_handler(nsock_pool nsp, nsock_event evt, void *mydata);
