/* $Id: ncat_ssl.h 12705 2009-03-26 23:15:49Z david $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_OPENSSL
#include <openssl/ssl.h>
#include <openssl/err.h>

extern SSL_CTX *setup_ssl_listen(void);

extern SSL *new_ssl(int fd);

#endif
